/*
 *  Copyright (c) 2012, Nigel Currie
 *	All rights reserved.
 *	
 *	Redistribution and use in source and binary forms, with or without modification,
 *	are permitted provided that the following conditions are met:
 *	
 *	- Redistributions of source code must retain the above copyright notice,
 *	  this list of conditions and the following disclaimer.
 *	- Redistributions in binary form must reproduce the above copyright notice,
 *	  this list of conditions and the following disclaimer in the documentation
 *	  and/or other materials provided with the distribution.
 *	- Neither the name of the <ORGANIZATION> nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software without
 *	  specific prior written permission.
 *	  
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *	IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 *	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 *	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 *	OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package examples;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.insolina.ydom.KeyValuePair;
import org.insolina.ydom.ScalarNode;
import org.insolina.ydom.SequenceNode;
import org.insolina.ydom.YamlDocument;
import org.insolina.ydom.YamlNode;

public class ShowExamples {
	public static void main(String[] args) {
		
		ShowExamples se = new ShowExamples();
		se.showMappingExample();
		se.showSequenceExample();
		se.showComplex2LevelExample();
		se.showComplex3LevelExample();
	}
	
	public void showMappingExample() {
		System.out.println("\n*** Simple Mapping Example ****************************");
		
		File f = loadFile("mapping.yaml");
		
		if(f != null) {
			YamlDocument doc = new YamlDocument(f);
		    showContents(f, doc);
			
		    System.out.println("\n\n-- Processed Data --");
		    System.out.println("MappingNode");
			List<YamlNode> keyValuePairs = doc.getChildren();
			for(YamlNode node : keyValuePairs) {
				KeyValuePair kvp = (KeyValuePair)node;
				System.out.println("\tKey=" + kvp.getKey() + ",Value=" + kvp.getValue().toString());
			}
			System.out.println();
		}
	}
	
	public void showSequenceExample() {
		System.out.println("\n*** Simple Sequence Example ****************************");
		
		File f = loadFile("sequence.yaml");
		
		if(f != null) {
			YamlDocument doc = new YamlDocument(f);
		    showContents(f, doc);
			
		    System.out.println("\n\n-- Processed Data --");
		    System.out.println("SequenceNode");
			List<YamlNode> items = doc.getChildren();
			for(YamlNode node : items) {
				System.out.println("\tValue=" + node.toString());
			}
			System.out.println();
		}
	}
	
	public void showComplex2LevelExample() {
		System.out.println("\n*** Complex 2 Level Example ****************************");
		
		File f = loadFile("complex2level.yaml");
		
		if(f != null) {
			YamlDocument doc = new YamlDocument(f);
		    showContents(f, doc);
			
		    System.out.println("\n\n-- Processed Data --");
		    System.out.println("SequenceNode");
			List<YamlNode> items = doc.getChildren();
			for(YamlNode node : items) {
				System.out.println("\tMappingNode");
				List<YamlNode> keyValuePairs = node.getChildren();
				for(YamlNode kvpnode : keyValuePairs) {
					KeyValuePair kvp = (KeyValuePair)kvpnode;
					System.out.println("\t\tKey=" + kvp.getKey() + ",Value=" + kvp.getValue().toString());
				}
			}
			System.out.println();
		}
	}
	
	public void showComplex3LevelExample() {
		System.out.println("\n*** Complex 3 Level Example ****************************");
		
		File f = loadFile("complex3level.yaml");
		
		if(f != null) {
			YamlDocument doc = new YamlDocument(f);
		    showContents(f, doc);
			
		    System.out.println("\n\n-- Processed Data --");
		    System.out.println("SequenceNode");
			List<YamlNode> items = doc.getChildren();
			for(YamlNode node : items) {
				System.out.println("\tMappingNode");
				List<YamlNode> keyValuePairs = node.getChildren();
				for(YamlNode kvpnode : keyValuePairs) {
					KeyValuePair kvp = (KeyValuePair)kvpnode;
					YamlNode kvpValue = kvp.getValue();
					if(kvpValue instanceof ScalarNode) {
						System.out.println("\t\tKey=" + kvp.getKey() + ",Value=" + kvp.getValue().toString());
					}
					else if(kvpValue instanceof SequenceNode) {
						System.out.println("\t\tKey=" + kvp.getKey() + ",");
						System.out.println("\t\tValue=SequenceNode");
						List<YamlNode> values = kvpValue.getChildren();		
						for(YamlNode val : values) {
							System.out.println("\t\t\tValue=" + val.toString());
						}
					}
				}
			}
			System.out.println();
		}
	}
	
	private File loadFile (String filename) {
		File file = null;
		
		try {
			final InputStream in = this.getClass().getResourceAsStream("/" + filename);
			file = toTempFile(in);
		}
		catch(Exception e) {
			throw new RuntimeException(e);
		}
		
		return file;
	}
	
	private void showContents(File f, YamlDocument doc) {
		System.out.println("\n-- File contents --");
		writeFileToStdout(f);
		System.out.println();
		
		System.out.println("\n-- YamlDom representation --");
		doc.print();
	}
	
	/*
	 * This convoluted method of reading in the yaml file is necessary
	 * because we're reading the file from the yamldom jar rather than 
	 * from the file store.
	 */
	private File toTempFile(InputStream in) throws IOException {  
		   
        File tempFile = File.createTempFile("tempFile", ".tmp");  
        tempFile.deleteOnExit();  
   
        FileOutputStream fout = null;  
   
        try {  
   
            fout = new FileOutputStream(tempFile);  
            int c;  
   
            while ((c = in.read()) != -1) {  
                fout.write(c);  
            }  
   
        }
        finally {  
            if (in != null) {  
                in.close();  
            }  
            if (fout != null) {  
                fout.close();  
            }  
        }  
        
        return tempFile;
    }  
	
	private void writeFileToStdout(File f) {
		try {
			FileInputStream fis = new FileInputStream(f);
	
			int oneByte;
			while ((oneByte = fis.read()) != -1) {
				System.out.write(oneByte);
			}
			System.out.flush();
		}
		catch(Exception e) {
			throw new RuntimeException(e);
		}
	}
}